@@ -22,6 +22,7 @@ class AllocTracer : public Engine {
     static u64 _interval;
     static volatile u64 _allocated_bytes;
 
+    static Error initialize();
     static void recordAllocation(void* ucontext, EventType event_type, uintptr_t rklass,
                                  uintptr_t total_size, uintptr_t instance_size);
 
@@ -38,7 +39,6 @@ class AllocTracer : public Engine {
         return "bytes";
     }
 
-    Error check(Arguments& args);
     Error start(Arguments& args);
     void stop();
 
