@@ -6,6 +6,10 @@
 package one.jfr.event;
 
 public class ExecutionSample extends Event {
+    // Synthetic thread state to distinguish samples converted from jdk.CPUTimeSample event.
+    // A small constant suitable for BitSet, does not clash with any existing thread state.
+    public static final int CPU_TIME_SAMPLE = 254;
+
     public final int threadState;
     public final int samples;
 
