@@ -58,6 +58,7 @@ static jclass _jfr_sync_class = NULL;
 static jmethodID _start_method;
 static jmethodID _stop_method;
 static jmethodID _box_method;
+static bool _jfr_starting = false;
 
 static const char* const SETTING_CSTACK[] = {NULL, "no", "fp", "dwarf", "lbr", "vm"};
 
@@ -665,6 +666,9 @@ class Recording {
         if (args._nativemem >= 0) {
             writeIntSetting(buf, T_MALLOC, "nativemem", args._nativemem);
         }
+        if (args._nativelock >= 0) {
+            writeIntSetting(buf, T_NATIVE_LOCK, "nativelock", args._nativelock);
+        }
 
         writeBoolSetting(buf, T_ALLOC_IN_NEW_TLAB, "enabled", args._alloc >= 0);
         writeBoolSetting(buf, T_ALLOC_OUTSIDE_TLAB, "enabled", args._alloc >= 0);
@@ -1070,6 +1074,7 @@ class Recording {
         buf->putVar32(call_trace_id);
         buf->putVar32(event->_thread_state);
         buf->putVar32(event->_samples);
+        buf->putVar64(event->_time_span);
         buf->put8(start, buf->offset() - start);
     }
 
@@ -1201,6 +1206,17 @@ class Recording {
         buf->put8(start, buf->offset() - start);
     }
 
+    void recordNativeLockSample(Buffer* buf, int tid, u32 call_trace_id, NativeLockEvent* event) {
+        int start = buf->skip(1);
+        buf->put8(T_NATIVE_LOCK);
+        buf->putVar64(event->_start_time);
+        buf->putVar64(event->_end_time - event->_start_time);
+        buf->putVar32(tid);
+        buf->putVar32(call_trace_id);
+        buf->putVar64(event->_address);
+        buf->put8(start, buf->offset() - start);
+    }
+
     void recordWindow(Buffer* buf, int tid, ProfilingWindow* event) {
         int start = buf->skip(1);
         buf->put8(T_WINDOW);
@@ -1387,7 +1403,9 @@ Error FlightRecorder::startMasterRecording(Arguments& args, const char* filename
     int event_mask = args.eventMask() |
                      ((args._jfr_options ^ JFR_SYNC_OPTS) << EVENT_MASK_SIZE);
 
+    __atomic_store_n(&_jfr_starting, true, __ATOMIC_RELEASE);
     env->CallStaticVoidMethod(_jfr_sync_class, _start_method, jfilename, jsettings, event_mask);
+    __atomic_store_n(&_jfr_starting, false, __ATOMIC_RELEASE);
 
     if (env->ExceptionCheck()) {
         env->ExceptionDescribe();
@@ -1441,6 +1459,9 @@ void FlightRecorder::recordEvent(int lock_index, int tid, u32 call_trace_id,
             case PARK_SAMPLE:
                 _rec->recordThreadPark(buf, tid, call_trace_id, (LockEvent*)event);
                 break;
+            case NATIVE_LOCK_SAMPLE:
+                _rec->recordNativeLockSample(buf, tid, call_trace_id, (NativeLockEvent*)event);
+                break;
             case PROFILING_WINDOW:
                 _rec->recordWindow(buf, tid, (ProfilingWindow*)event);
                 break;
@@ -1473,3 +1494,7 @@ void FlightRecorder::recordLog(LogLevel level, const char* message, size_t len)
 
     _rec_lock.unlockShared();
 }
+
+bool FlightRecorder::isJfrStarting() {
+    return __atomic_load_n(&_jfr_starting, __ATOMIC_ACQUIRE);
+}
