@@ -25,6 +25,37 @@ enum ThreadState {
     THREAD_SLEEPING
 };
 
+struct ProcessInfo {
+    int pid = 0;
+    int ppid = 0;
+    char name[16];           // Process name from /proc/{pid}/stats
+    char cmdline[2048];      // Command line from /proc/{pid}/cmdline
+    unsigned int uid = 0;    // User ID
+    unsigned char state = 0; // Process state (R, S, D, Z, T, etc.)
+    u64 start_time = 0;      // Process start time (milliseconds since epoch)
+
+    // CPU & thread stats
+    float cpu_user = 0;    // User CPU time (seconds)
+    float cpu_system = 0;  // System CPU time (seconds)
+    float cpu_percent = 0; // CPU utilization percentage
+    int threads = 0;       // Number of threads
+
+    // Memory stats (in bytes)
+    u64 vm_size = 0;   // Total virtual memory size
+    u64 vm_rss = 0;    // Resident memory size
+    u64 rss_anon = 0;  // Resident anonymous memory
+    u64 rss_files = 0; // Resident file mappings
+    u64 rss_shmem = 0; // Resident shared memory
+
+    // Page fault stats
+    u64 minor_faults = 0; // Minor page faults (no I/O required)
+    u64 major_faults = 0; // Major page faults (I/O required)
+
+    // I/O stats
+    u64 io_read = 0;  // KB read from storage
+    u64 io_write = 0; // KB written to storage
+};
+
 
 class ThreadList {
   protected:
@@ -65,11 +96,13 @@ class OS {
   public:
     static const size_t page_size;
     static const size_t page_mask;
+    static const long clock_ticks_per_sec;
 
     static u64 nanotime();
     static u64 micros();
     static u64 processStartTime();
     static void sleep(u64 nanos);
+    static void uninterruptibleSleep(u64 nanos, volatile bool* flag);
     static u64 overrun(siginfo_t* siginfo);
 
     static u64 hton64(u64 x);
@@ -106,6 +139,12 @@ class OS {
     static int mprotect(void* addr, size_t size, int prot);
 
     static bool checkPreloaded();
+
+    static u64 getSystemBootTime();
+    static u64 getRamSize();
+    static int getProcessIds(int* pids, int max_pids);
+    static bool getBasicProcessInfo(int pid, ProcessInfo* info);
+    static bool getDetailedProcessInfo(ProcessInfo* info);
 };
 
 #endif // _OS_H
