@@ -181,7 +181,7 @@ struct PerfEventType {
 
     enum {
         IDX_CPU = 0,
-        IDX_PREDEFINED = 12,
+        IDX_PREDEFINED = 13,
         IDX_RAW,
         IDX_PMU,
         IDX_BREAKPOINT,
@@ -472,6 +472,7 @@ PerfEventType PerfEventType::AVAILABLE_EVENTS[] = {
     {"branch-instructions",   1000000, PERF_TYPE_HARDWARE, PERF_COUNT_HW_BRANCH_INSTRUCTIONS},
     {"branch-misses",            1000, PERF_TYPE_HARDWARE, PERF_COUNT_HW_BRANCH_MISSES},
     {"bus-cycles",            1000000, PERF_TYPE_HARDWARE, PERF_COUNT_HW_BUS_CYCLES},
+    {"ref-cycles",            1000000, PERF_TYPE_HARDWARE, PERF_COUNT_HW_REF_CPU_CYCLES},
 
     {"L1-dcache-load-misses", 1000000, PERF_TYPE_HW_CACHE, LOAD_MISS(PERF_COUNT_HW_CACHE_L1D)},
     {"LLC-load-misses",          1000, PERF_TYPE_HW_CACHE, LOAD_MISS(PERF_COUNT_HW_CACHE_LL)},
@@ -547,6 +548,7 @@ PerfEventType* PerfEvents::_event_type = NULL;
 int PerfEvents::_ioc_enable;
 bool PerfEvents::_alluser;
 bool PerfEvents::_kernel_stack;
+bool PerfEvents::_use_perf_mmap;
 bool PerfEvents::_record_cpu;
 int PerfEvents::_target_cpu;
 
@@ -634,7 +636,7 @@ int PerfEvents::createForThread(int tid) {
     }
 
     void* page = NULL;
-    if (_kernel_stack || _cstack == CSTACK_DEFAULT || _cstack == CSTACK_LBR) {
+    if (_use_perf_mmap) {
         page = mmap(NULL, 2 * OS::page_size, PROT_READ | PROT_WRITE, MAP_SHARED, fd, 0);
         if (page == MAP_FAILED) {
             Log::warn("perf_event mmap failed: %s", strerror(errno));
@@ -764,59 +766,6 @@ const char* PerfEvents::units() {
     return _event_type == NULL || strcmp(_event_type->name, "cpu-clock") == 0 ? "ns" : "total";
 }
 
-Error PerfEvents::check(Arguments& args) {
-    PerfEventType* event_type = PerfEventType::forName(args._event);
-    if (event_type == NULL) {
-        return Error("Unsupported event type");
-    } else if (event_type->counter_arg > 4) {
-        return Error("Only arguments 1-4 can be counted");
-    }
-
-    if (!setupThreadHook()) {
-        return Error("Could not set pthread hook");
-    }
-
-    struct perf_event_attr attr = {0};
-    attr.size = sizeof(attr);
-    attr.type = event_type->type;
-
-    if (attr.type == PERF_TYPE_BREAKPOINT) {
-        attr.bp_type = event_type->config;
-    } else {
-        attr.config = event_type->config;
-    }
-    attr.config1 = event_type->config1;
-    attr.config2 = event_type->config2;
-
-    attr.sample_period = event_type->default_interval;
-    attr.sample_type = PERF_SAMPLE_CALLCHAIN;
-    attr.disabled = 1;
-
-    if (args._alluser) {
-        attr.exclude_kernel = 1;
-    }
-
-#ifdef PERF_ATTR_SIZE_VER5
-    if (args._cstack == CSTACK_LBR) {
-        attr.sample_type |= PERF_SAMPLE_BRANCH_STACK | PERF_SAMPLE_REGS_USER;
-        attr.branch_sample_type = PERF_SAMPLE_BRANCH_USER | PERF_SAMPLE_BRANCH_CALL_STACK;
-        attr.sample_regs_user = 1ULL << PERF_REG_PC;
-    }
-#endif
-
-    if (args._record_cpu) {
-        attr.sample_type |= PERF_SAMPLE_CPU;
-    }
-
-    int fd = syscall(__NR_perf_event_open, &attr, 0, args._target_cpu, -1, 0);
-    if (fd == -1) {
-        return Error(strerror(errno));
-    }
-
-    close(fd);
-    return Error::OK;
-}
-
 Error PerfEvents::start(Arguments& args) {
     _event_type = PerfEventType::forName(args._event);
     if (_event_type == NULL) {
@@ -850,6 +799,7 @@ Error PerfEvents::start(Arguments& args) {
         // Automatically switch on alluser for non-CPU events, if kernel profiling is unavailable
         _alluser = strcmp(args._event, EVENT_CPU) != 0 && !supported();
     }
+    _use_perf_mmap = _kernel_stack || _cstack == CSTACK_DEFAULT || _cstack == CSTACK_LBR || _record_cpu;
 
     if (strcmp(_event_type->name, "cpu-clock") == 0 && hasPerfEventRefreshBug()) {
         Log::debug("Enable workaround for PERF_EVENT_IOC_REFRESH bug");
