@@ -12,13 +12,13 @@
 #include "vmStructs.h"
 
 
-const uintptr_t SAME_STACK_DISTANCE = 8192;
 const uintptr_t MAX_WALK_SIZE = 0x100000;
 const intptr_t MAX_FRAME_SIZE = 0x40000;
 const intptr_t MAX_INTERPRETER_FRAME_SIZE = 0x1000;
 const intptr_t DEAD_ZONE = 0x1000;
 
 static ucontext_t empty_ucontext{};
+static jmp_buf* crash_protection_ctx[CONCURRENCY_LEVEL];
 
 
 static inline bool aligned(uintptr_t ptr) {
@@ -29,10 +29,6 @@ static inline bool inDeadZone(const void* ptr) {
     return ptr < (const void*)DEAD_ZONE || ptr > (const void*)-DEAD_ZONE;
 }
 
-static inline bool sameStack(void* hi, void* lo) {
-    return (uintptr_t)hi - (uintptr_t)lo < SAME_STACK_DISTANCE;
-}
-
 // AArch64: on Linux, frame link is stored at the top of the frame,
 // while on macOS, frame link is at the bottom.
 static inline uintptr_t defaultSenderSP(uintptr_t sp, uintptr_t fp) {
@@ -173,7 +169,7 @@ int StackWalker::walkDwarf(void* ucontext, const void** callchain, int max_depth
             break;
         }
 
-        const void* prev_pc = pc; 
+        const void* prev_pc = pc;
         if (f->fp_off & DW_PC_OFFSET) {
             pc = (const char*)pc + (f->fp_off >> 1);
         } else {
@@ -206,75 +202,66 @@ int StackWalker::walkDwarf(void* ucontext, const void** callchain, int max_depth
     return depth;
 }
 
-int StackWalker::walkVM(void* ucontext, ASGCT_CallFrame* frames, int max_depth,
+int StackWalker::walkVM(void* ucontext, ASGCT_CallFrame* frames, int max_depth, int lock_index,
                         StackWalkFeatures features, EventType event_type) {
+    const void* pc;
+    uintptr_t fp;
+    uintptr_t sp;
+    uintptr_t bottom = (uintptr_t)&sp + MAX_WALK_SIZE;
+
+    StackFrame frame(ucontext ? ucontext : &empty_ucontext);
     if (ucontext == NULL) {
-        return walkVM(&empty_ucontext, frames, max_depth, features, event_type,
-                      callerPC(), (uintptr_t)callerSP(), (uintptr_t)callerFP());
+        pc = callerPC();
+        fp = (uintptr_t)callerFP();
+        sp = (uintptr_t)callerSP();
     } else {
-        StackFrame frame(ucontext);
-        return walkVM(ucontext, frames, max_depth, features, event_type,
-                      (const void*)frame.pc(), frame.sp(), frame.fp());
+        pc = (const void*)frame.pc();
+        fp = frame.fp();
+        sp = frame.sp();
     }
-}
-
-int StackWalker::walkVM(void* ucontext, ASGCT_CallFrame* frames, int max_depth, JavaFrameAnchor* anchor, EventType event_type) {
-    uintptr_t sp = anchor->lastJavaSP();
-    if (sp == 0) {
-        return 0;
-    }
-
-    uintptr_t fp = anchor->lastJavaFP();
-    if (fp == 0) {
-        fp = sp;
-    }
-
-    const void* pc = anchor->lastJavaPC();
-    if (pc == NULL) {
-        pc = ((const void**)sp)[-1];
-    }
-
-    StackWalkFeatures no_features{};
-    return walkVM(ucontext, frames, max_depth, no_features, event_type, pc, sp, fp);
-}
-
-int StackWalker::walkVM(void* ucontext, ASGCT_CallFrame* frames, int max_depth,
-                        StackWalkFeatures features, EventType event_type,
-                        const void* pc, uintptr_t sp, uintptr_t fp) {
-    StackFrame frame(ucontext);
-    uintptr_t bottom = (uintptr_t)&frame + MAX_WALK_SIZE;
 
     Profiler* profiler = Profiler::instance();
     int bcp_offset = InterpreterFrame::bcp_offset();
 
-    jmp_buf crash_protection_ctx;
-    VMThread* vm_thread = VMThread::current();
-    void* saved_exception = vm_thread != NULL ? vm_thread->exception() : NULL;
+    jmp_buf current_ctx;
+    crash_protection_ctx[lock_index] = &current_ctx;
 
     // Should be preserved across setjmp/longjmp
     volatile int depth = 0;
 
-    if (vm_thread != NULL) {
-        vm_thread->exception() = &crash_protection_ctx;
-        if (setjmp(crash_protection_ctx) != 0) {
-            vm_thread->exception() = saved_exception;
-            if (depth < max_depth) {
-                fillFrame(frames[depth++], BCI_ERROR, "break_not_walkable");
-            }
-            return depth;
+    if (setjmp(current_ctx) != 0) {
+        crash_protection_ctx[lock_index] = NULL;
+        if (depth < max_depth) {
+            fillFrame(frames[depth++], BCI_ERROR, "break_not_walkable");
         }
+        return depth;
     }
 
     // Show extended frame types and stub frames for execution-type events
     bool details = event_type <= MALLOC_SAMPLE || features.mixed;
 
     JavaFrameAnchor* anchor = NULL;
-    if (details && vm_thread != NULL && vm_thread->isJavaThread()) {
-        anchor = vm_thread->anchor();
+    VMThread* vm_thread = VMThread::current();
+    if (vm_thread != NULL && vm_thread->isJavaThread()) {
+        // For simple stack traces (e.g. for allocation profiling)
+        // jump directly to the first Java frame
+        if (details) {
+            anchor = vm_thread->anchor();
+        } else if (!vm_thread->anchor()->restoreFrame(pc, sp, fp)) {
+            return 0;
+        }
     }
 
     unwind_loop:
+    uintptr_t prev_sp = sp;
     while (depth < max_depth) {
+        // As an extra safety measure, verify stack pointer invariants on every iteration
+        if (sp < prev_sp || sp >= bottom || !aligned(sp)) {
+            fillFrame(frames[depth++], BCI_ERROR, "break_stack_range");
+            break;
+        }
+        prev_sp = sp;
+
         if (CodeHeap::contains(pc)) {
             NMethod* nm = CodeHeap::findNMethod(pc);
             if (nm == NULL) {
@@ -385,15 +372,10 @@ int StackWalker::walkVM(void* ucontext, ASGCT_CallFrame* frames, int max_depth,
                     fillFrame(frames[depth++], BCI_ERROR, "break_entry_frame");
                     break;
                 }
-                uintptr_t prev_sp = sp;
                 if (!next_anchor->getFrame(pc, sp, fp)) {
                     // End of Java stack
                     break;
                 }
-                if (sp < prev_sp || sp >= bottom || !aligned(sp)) {
-                    fillFrame(frames[depth++], BCI_ERROR, "break_entry_frame");
-                    break;
-                }
                 continue;
             } else {
                 if (features.vtable_target && nm->isVTableStub() && depth == 0) {
@@ -428,8 +410,8 @@ int StackWalker::walkVM(void* ucontext, ASGCT_CallFrame* frames, int max_depth,
             const char* method_name = profiler->findNativeMethod(pc);
             char mark;
             if (method_name != NULL && (mark = NativeFunc::mark(method_name)) != 0) {
-                if (mark == MARK_ASYNC_PROFILER && event_type == MALLOC_SAMPLE) {
-                    // Skip all internal frames above malloc_hook functions, leave the hook itself
+                if (mark == MARK_ASYNC_PROFILER && (event_type == MALLOC_SAMPLE || event_type == NATIVE_LOCK_SAMPLE)) {
+                    // Skip all internal frames above hook functions, leave the hook itself
                     depth = 0;
                 } else if (mark == MARK_COMPILER_ENTRY && features.comp_task && vm_thread != NULL) {
                     // Insert current compile task as a pseudo Java frame
@@ -443,7 +425,6 @@ int StackWalker::walkVM(void* ucontext, ASGCT_CallFrame* frames, int max_depth,
             fillFrame(frames[depth++], BCI_NATIVE_FRAME, method_name);
         }
 
-        uintptr_t prev_sp = sp;
         CodeCache* cc = profiler->findLibraryByAddress(pc);
         FrameDesc* f = cc != NULL ? cc->findFrameDesc(pc) : &FrameDesc::default_frame;
 
@@ -469,7 +450,7 @@ int StackWalker::walkVM(void* ucontext, ASGCT_CallFrame* frames, int max_depth,
             break;
         }
 
-        const void* prev_pc = pc; 
+        const void* prev_pc = pc;
         if (f->fp_off & DW_PC_OFFSET) {
             pc = (const char*)pc + (f->fp_off >> 1);
         } else {
@@ -488,9 +469,6 @@ int StackWalker::walkVM(void* ucontext, ASGCT_CallFrame* frames, int max_depth,
             if (EMPTY_FRAME_SIZE == 0 && cfa_off == 0 && f->fp_off != DW_SAME_FP) {
                 // AArch64 default_frame
                 sp = defaultSenderSP(sp, fp);
-                if (sp < prev_sp || sp >= bottom || !aligned(sp)) {
-                    break;
-                }
             }
         }
 
@@ -507,19 +485,28 @@ int StackWalker::walkVM(void* ucontext, ASGCT_CallFrame* frames, int max_depth,
         goto unwind_loop;
     }
 
-    if (vm_thread != NULL) vm_thread->exception() = saved_exception;
+    crash_protection_ctx[lock_index] = NULL;
 
     return depth;
 }
 
 void StackWalker::checkFault() {
-    if (VMThread::key() < 0) {
-        // JVM has not been loaded or VMStructs have not been initialized yet
-        return;
+    // Search for a crash protection context located on the current thread stack.
+    // Since one thread may use multiple contexts because of simultaneous profiling engines,
+    // we need to walk through all of them and find the nearest one (i.e. the most recent).
+    jmp_buf* nearest_ctx = NULL;
+    uintptr_t stack_distance = 32768;  // maximum allowed stack distance
+    const uintptr_t current_sp = (uintptr_t)&nearest_ctx;
+
+    for (int i = 0; i < CONCURRENCY_LEVEL; i++) {
+        jmp_buf* ctx = crash_protection_ctx[i];
+        if ((uintptr_t)ctx - current_sp < stack_distance) {
+            nearest_ctx = ctx;
+            stack_distance = (uintptr_t)ctx - current_sp;
+        }
     }
 
-    VMThread* vm_thread = VMThread::current();
-    if (vm_thread != NULL && sameStack(vm_thread->exception(), &vm_thread)) {
-        longjmp(*(jmp_buf*)vm_thread->exception(), 1);
+    if (nearest_ctx != NULL) {
+        longjmp(*nearest_ctx, 1);
     }
 }
