@@ -16,6 +16,7 @@ TEST_CASE(Parse_all_mode_no_override) {
     ASSERT_EQ(args._alloc, 0);
     ASSERT_EQ(args._nativemem, DEFAULT_ALLOC_INTERVAL);
     ASSERT_EQ(args._lock, DEFAULT_LOCK_INTERVAL);
+    ASSERT_EQ(args._nativelock, DEFAULT_LOCK_INTERVAL);
     ASSERT_EQ(args._live, true);
     const char* expected_event = OS::isLinux() ? EVENT_CPU : NULL;
     ASSERT_EQ(args._event, expected_event);
@@ -32,13 +33,14 @@ TEST_CASE(Parse_all_mode_event_override) {
     ASSERT_EQ(args._alloc, 0);
     ASSERT_EQ(args._nativemem, DEFAULT_ALLOC_INTERVAL);
     ASSERT_EQ(args._lock, DEFAULT_LOCK_INTERVAL);
+    ASSERT_EQ(args._nativelock, DEFAULT_LOCK_INTERVAL);
     ASSERT_EQ(args._live, true);
     ASSERT_EQ(args._proc, OS::isLinux() ? DEFAULT_PROC_INTERVAL : -1);
 }
 
 TEST_CASE(Parse_all_mode_event_and_threshold_override) {
     Arguments args;
-    char argument[] = "start,all,event=cycles,nativemem=10,lock=100,alloc=1000,wall=10000,proc=10,file=%f.jfr";
+    char argument[] = "start,all,event=cycles,nativemem=10,lock=100,alloc=1000,wall=10000,proc=10,nativelock=100000,file=%f.jfr";
     Error error = args.parse(argument);
     ASSERT_EQ(args._all, true);
     ASSERT_EQ(args._event, "cycles");
@@ -46,13 +48,14 @@ TEST_CASE(Parse_all_mode_event_and_threshold_override) {
     ASSERT_EQ(args._alloc, 1000);
     ASSERT_EQ(args._nativemem, 10);
     ASSERT_EQ(args._lock, 100);
+    ASSERT_EQ(args._nativelock, 100000);
     ASSERT_EQ(args._live, true);
     ASSERT_EQ(args._proc, 10);
 }
 
 TEST_CASE(Parse_override_before_all_mode) {
     Arguments args;
-    char argument[] = "start,event=cycles,nativemem=10,lock=100,alloc=1000,proc=10,all,file=%f.jfr";
+    char argument[] = "start,event=cycles,nativemem=10,lock=100,alloc=1000,proc=10,nativelock=1000,all,file=%f.jfr";
     Error error = args.parse(argument);
     ASSERT_EQ(args._all, true);
     ASSERT_EQ(args._event, "cycles");
@@ -60,6 +63,7 @@ TEST_CASE(Parse_override_before_all_mode) {
     ASSERT_EQ(args._alloc, 1000);
     ASSERT_EQ(args._nativemem, 10);
     ASSERT_EQ(args._lock, 100);
+    ASSERT_EQ(args._nativelock, 1000);
     ASSERT_EQ(args._live, true);
     ASSERT_EQ(args._proc, 10);
 }
