@@ -12,9 +12,7 @@ public class ComptaskTests {
         mainClass = Main.class,
         agentArgs = "start,features=comptask,collapsed,interval=1ms,file=%f",
         jvmArgs = "-Xcomp",
-        jvm = Jvm.HOTSPOT,
-        // TODO: remove version filter after the fix for JDK-8367689 is released
-        jvmVer = {8, 24}
+        jvm = Jvm.HOTSPOT_C2
     )
     public void testCompTask(TestProcess p) throws Exception {
         Output out = p.waitForExit("%f");
