@@ -28,7 +28,7 @@ public class NativememTests {
     private static final int POSIX_MEMALIGN_SIZE = 30000193;
     private static final int ALIGNED_ALLOC_SIZE = 30002016;
 
-    @Test(mainClass = CallsMallocCalloc.class, agentArgs = "start,nativemem,total,collapsed,file=%f", args = "once")
+    @Test(mainClass = CallsMallocCalloc.class, agentArgs = "start,nativemem,total,cstack=fp,collapsed,file=%f", args = "once", nameSuffix = "FP")
     @Test(mainClass = CallsMallocCalloc.class, agentArgs = "start,nativemem,total,cstack=vm,collapsed,file=%f", args = "once", nameSuffix = "VM")
     public void canAgentTraceMallocCalloc(TestProcess p) throws Exception {
         Output out = p.waitForExit("%f");
